/**
 * Custom javascript for Sector ToC.
 */
(function ($) {
  'use strict';

  /**
   * Tracking headings to highlight toc entries while scrolling.
   * Ignore scroll events while scrolling due to toc link click.
   */
  Drupal.behaviors.sector_toc = {
    attach: function (context, settings) {
      var isScrolling;

      // Add event handler to toc links.
      $('.toc-tree a').each(function() {
        $(this).on('click', function() {
          logger('clicked ' + $(this).attr('href'));
          // Set flag on .toc-tree to ignore updates while we scroll to target.
          $('.toc-tree').data('scroll-ignore', 1);
        });
      });

      function tocLocationHashChange() {
        var hash = location.hash;
        logger("tocLocationHashChange() hash=" + hash);

        // If hash matches a toc entry...
        if ($('.toc-tree a[href$="' + hash + '"]').length) {
          setActive('.toc_tree', hash);
        }
        else {
          logger('tocLocationHashChange() no hash==toc match, want to ignore scroll?');
          // Suppress URL hash changes while we scroll to target.
          $('.toc-tree').data('scroll-ignore', 1);
          // Find parent chunker section, then make h2 active.
          let h2 = $(hash).parents('div.chunker-section').find('h2');
          logger('nearest h2=' + h2.html());
          let h2_id = h2.attr('id');
          // Make toc with matching href 'active'.
          setActive('.toc_tree', '#' + h2_id);
        }
      }

      // Set hyperlink and parent list item active in ToC.
      function setActive(toc_selector, hash) {
        // Remove active from any non-matched link.
        $('.toc-tree a[href!="' + hash + '"]').removeClass('active');
        // Remove active class from any active list items.
        $('.toc-tree li.active').each(function () {
          $(this).removeClass('active');
        });
        // Add active class to matched anchor, and parent list item.
        $('.toc-tree a[href$="' + hash + '"]')
            .addClass('active')
            .parents('li')
            .each(function () {
              $(this).addClass('active')
            });
      }

      // @todo: what is correct Drupal way to bind to window events?
      window.addEventListener("hashchange", tocLocationHashChange, false);

      // On page load, do we have a URL fragment?
      // e.g. arriving from bookmark, or refresh existing page?
      if (location.hash) {
        logger('detected hash in url, triggering local hashchange event')
        tocLocationHashChange();
      }

      let headings = document.querySelectorAll('.node h2, .node h3');

      let options = {
        // root: document.querySelector('.node'),
        rootMargin: '0px',
        threshold: 1.0
      }

      let observer = new IntersectionObserver(handleObserver, options);

      headings.forEach(heading => {
        observer.observe(heading);
      });

      function handleObserver(entries, observer){
        entries.forEach(entry=>{
          let heading_id = entry.target.getAttribute('id');
          if (entry.isIntersecting && entry.intersectionRatio === 1) {
            logger('intersecting heading_id=' + heading_id);
            if (heading_id !== null) {
              const location = window.location.toString().split('#')[0];
              if (heading_id !== location.hash) {
                // Are we ignoring scroll (e.g. relocating to toc link click).
                if ($('.toc-tree').data('scroll-ignore') !== 1) {
                  logger('observer, setting id=' + heading_id);
                  history.replaceState(null, null, location + '#' + heading_id);
                  window.dispatchEvent(new HashChangeEvent("hashchange"));
                }
                else {
                  logger('scroll, scroll-ignore');
                }
              }
            }
          }
        });
      }

      // Detect completion of scrolling.
      document.addEventListener('scroll', (e) => {
        // console.log('sector_toc: scroll');
        // Reset timeout while scrolling.
        window.clearTimeout(isScrolling);
        isScrolling = setTimeout(function(){
          logger('I think scrolling has stopped...');
          $('.toc-tree').data('scroll-ignore', 0);
        }, 600);
        // Perhaps better would be to bind intersection observer to element
        // with id being sought via URL so we know when we have arrived...
      });

      function logger(message) {
        const prefix = 'sector_toc: ';
        const debug = false;

        if (debug) {
          if (message.isObject) {
            console.log(prefix, message);
          }
          else {
            console.log(prefix + message);
          }
        }
      }
    }
  }

})(jQuery);
